@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.03_2
+Version 0.03_3
 
 =cut
 
-our $VERSION = '0.03_2';
+our $VERSION = '0.03_3';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -98,6 +98,10 @@ As an example, some card representations:
 	4D - 4 of Diamonds
 	KH - King of Hearts
 
+Still, you can add whichever cards you want to the deck:
+
+	$deck->put( 'Joker' ); # place a Joker on top of the deck
+
 
 =head1 SUBROUTINES/METHODS
 
@@ -176,7 +180,7 @@ sub restart {
 }
 
 
-=head3 deck_size
+=head3 size, deck_size
 
 Returns the size of the deck.
 
@@ -184,12 +188,33 @@ Returns the size of the deck.
 
 =cut
 
+sub size {
+	my $self = shift;
+	return $self->deck_size;
+}
+
 sub deck_size {
 	my $self = shift;
 	return scalar @{$self->{'deck'}};
 }
 
 
+=head3 original_size
+
+Returns the original size of the deck.
+
+	if ($size < 0.5 * $deck->original_size) {
+		# if the deck has been exausted by over 50%
+	}
+
+=cut
+
+sub original_size {
+	my $self = shift;
+	return scalar @{$self->{'original'}};
+}
+
+
 =head3 get_deck
 
 Returns the deck (a reference to a list of strings).
@@ -240,9 +265,7 @@ sub turn {
 
 	$self->{'orientation'} = $self->orientation eq 'down' ? 'up' : 'down';
 
-	$self->_set_deck( reverse @{$self->get_deck} );
-
-	return $self;
+	return $self->_set_deck( reverse @{$self->get_deck} );
 }
 
 
@@ -264,7 +287,13 @@ You can specify how many times you want to go through the deck (which is basical
 
 sub overhand_shuffle {
 	my $self  = shift;
-	my $times = shift || 1;
+	my $times = shift;
+
+	if (not defined $times) {
+		$times = 1;
+	}
+
+	return $self if $times < 1;
 
 	$self->_packet_transfer( 1, 10 );
 
@@ -400,6 +429,7 @@ You can also decide where to cut the deck for the shuffle:
 # TODO: add an option for an out-shuffle
 # TODO: add an option to control top or bottom stock
 # TODO: when dropping cards, should we favor numbers 2 and 3?
+# TODO: with a lot of cards, the riffle should be done breaking the deck in piles
 sub riffle_shuffle {
 	my $self  = shift;
 	my $depth = shift;
@@ -626,6 +656,7 @@ Places a pile of cards on top of the deck.
 
 =cut
 
+# TODO place_on_top and put share similar code; review
 sub place_on_top {
 	my $self = shift;
 	my @pile = @_;
@@ -1039,9 +1070,97 @@ sub take_random {
 	return $self->remove( _rand( $lower_limit, $upper_limit ) );
 }
 
+=head3 dribble
+
+Dribble the cards (usually to select one, which could either be the last one to fall or the one that would be next).
+
+	# dribble cards onto $pile
+	$deck->dribble( $pile );
+
+	# same thing, but declaring $pile
+    $pile = $deck->dribble;
+
+    # dribble to position 10 (in a 52 card deck, 42 cards would fall)
+    $deck->dribble( 10 );
+
+    # dribble 10 cards
+    $deck->dribble( -10 );
+
+    # dribble to position between 10 and 20
+    $deck->dribble( 10, 20 );
+
+    # dribble to position between 10th from the top and 10th from the bottom
+    $deck->dribble( 10, -10 );
+
+=cut
+
+# TODO: what happens if you're dribbling and have no cards?
+sub dribble {
+	my $self   = shift;
+	my $params = _parse_params( @_ );
+
+	my $has_destination = @{$params->{'places'}};
+	my $destination = $has_destination ?
+						$params->{'places'}[0] :
+						Games::Cards::ShuffleTrack->new( 'empty' );
+
+	my ($lower_limit, $upper_limit) = $self->_fix_limits( @{$params->{'numbers'}} );
+
+	if ( defined $lower_limit ) {
+		$lower_limit = $self->size - $lower_limit;
+		$upper_limit = defined $upper_limit ?
+						$self->size - $upper_limit :
+						$lower_limit;
+	}
+	else {
+		# FIXME: this means we can't dribble if we have just 5 cards
+		$lower_limit = 5; # minimum amount of cards to hold back
+		$upper_limit = $self->size - 5; # minimum amount of cards to dribble
+	}
+
+	$self->turn;
+	my $transfer = $self->cut_to( $lower_limit, $upper_limit );
+	$transfer->turn;
+	$transfer->move_to( $destination );
+
+	return $has_destination ? $self : $destination;
+}
 
 # subroutines
 
+# TODO: fix limits in other methods (just being used in dribble)
+sub _fix_limits {
+	my $self = shift;
+	my @limits;
+
+	while ( my $limit = shift ) {
+		push @limits, $limit < 0 ? $self->size + $limit : $limit;
+	}
+
+	return @limits;
+}
+
+# TODO: use this for every method (just being used in dribble)
+sub _parse_params {
+	my $params = {
+		numbers => [],
+		places  => [],
+		options => [],
+	};
+	while (my $param = shift) {
+		if (looks_like_number($param)) {
+			push @{$params->{'numbers'}}, $param;
+		}
+		elsif (ref $param eq 'Games::Cards::ShuffleTrack') {
+			push @{$params->{'places'}}, $param;
+		}
+		else {
+			push @{$params->{'options'}}, $param;
+		}
+	}
+	return $params;
+}
+
 sub _set_deck {
 	my $self = shift;
 	return $self->{'deck'} = [@_];
